@@ -32,7 +32,9 @@ from construct import (
     Byte,
     BytesInteger,
     Embedded,
+    ExprAdapter,
     Flag,
+    Float32b,
     Int8sl,
     Int8ul,
     Int16sl,
@@ -40,9 +42,9 @@ from construct import (
     Int24ul,
     Int32ul,
     PaddedString,
-    Probe,
     Struct,
     Switch,
+    obj_,
     this,
 )
 
@@ -79,7 +81,7 @@ class PropertyID(IntEnum):
     DEVICE_SOFTWARE_REVISION = 0x001A
     DEVICE_UNDER_TEMPERATURE_EVENT_STATISTICS = 0x001B
     INDOOR_AMBIENT_TEMPERATURE_STATISTICAL_VALUES = 0x001C
-    INITIAL_CIE_1931_CHROMATICITY_COORDINATES = 0x001D
+    INITIAL_CIE1931_CHROMATICITY_COORDINATES = 0x001D
     INITIAL_CORRELATED_COLOR_TEMPERATURE = 0x001E
     INITIAL_LUMINOUS_FLUX = 0x001F
     INITIAL_PLANCKIAN_DISTANCE = 0x0020
@@ -130,7 +132,7 @@ class PropertyID(IntEnum):
     PRESENCE_DETECTED = 0x004D
     PRESENT_AMBIENT_LIGHT_LEVEL = 0x004E
     PRESENT_AMBIENT_TEMPERATURE = 0x004F
-    PRESENT_CIE_1931_CHROMATICITY_COORDINATES = 0x0050
+    PRESENT_CIE1931_CHROMATICITY_COORDINATES = 0x0050
     PRESENT_CORRELATED_COLOR_TEMPERATURE = 0x0051
     PRESENT_DEVICE_INPUT_POWER = 0x0052
     PRESENT_DEVICE_OPERATING_EFFICIENCY = 0x0053
@@ -170,6 +172,8 @@ class PropertyID(IntEnum):
 
 
 class TimeExponential8Validator(Adapter):
+    _subcon = Float32b
+
     def _decode(self, obj, content, path):
         return round(pow(1.1, obj - 64), 4) if obj else 0
 
@@ -178,17 +182,34 @@ class TimeExponential8Validator(Adapter):
 
 
 class DateValidator(Adapter):
+    _subcon = Int32ul
     EPOCH = datetime(1970, 1, 1)
 
     def _decode(self, obj, content, path):
-        return None if obj == 0x0 else self.EPOCH + timedelta(days=obj)
+        if obj is None:
+            return 0
+
+        return self.EPOCH + timedelta(days=obj)
 
     def _encode(self, obj, content, path):
-        return 0x0 if obj is None else (obj - self.EPOCH).days
+        if obj is None:
+            return 0
+
+        if isinstance(obj, datetime):
+            return (obj - self.EPOCH).days
+
+        return obj
 
 
 def FixedString(size):
-    return PaddedString(size, "utf8")
+    def decode_bytes(obj, context):
+        return obj.decode() if isinstance(obj, bytes) else obj
+
+    return ExprAdapter(
+        PaddedString(size, "utf8"),
+        obj_,
+        decode_bytes,
+    )
 
 
 # fmt: off
@@ -263,7 +284,6 @@ Voltage = Struct(
 AverageVoltage = Struct(
     "voltage_value" / DefaultCountValidator(Int16ul, resolution=1/64),
     "sensing_duration" / TimeExponential8,
-    Probe(this)
 )
 
 VoltageRange = Struct(
@@ -504,7 +524,7 @@ PropertyDict = {
     PropertyID.DEVICE_SOFTWARE_REVISION: FixedString(8),
     PropertyID.DEVICE_UNDER_TEMPERATURE_EVENT_STATISTICS: EventStatistics,
     PropertyID.INDOOR_AMBIENT_TEMPERATURE_STATISTICAL_VALUES: Temperature8Statistics,
-    PropertyID.INITIAL_CIE_1931_CHROMATICITY_COORDINATES: ChromaticityCoordinates,
+    PropertyID.INITIAL_CIE1931_CHROMATICITY_COORDINATES: ChromaticityCoordinates,
     PropertyID.INITIAL_CORRELATED_COLOR_TEMPERATURE: CorrelatedColorTemperature,
     PropertyID.INITIAL_LUMINOUS_FLUX: LuminousFlux,
     PropertyID.INITIAL_PLANCKIAN_DISTANCE: ChromaticDistanceFromPlanckian,
@@ -555,7 +575,7 @@ PropertyDict = {
     PropertyID.PRESENCE_DETECTED: Presence,
     PropertyID.PRESENT_AMBIENT_LIGHT_LEVEL: Illuminance,
     PropertyID.PRESENT_AMBIENT_TEMPERATURE: Temperature8,
-    PropertyID.PRESENT_CIE_1931_CHROMATICITY_COORDINATES: ChromaticityCoordinates,
+    PropertyID.PRESENT_CIE1931_CHROMATICITY_COORDINATES: ChromaticityCoordinates,
     PropertyID.PRESENT_CORRELATED_COLOR_TEMPERATURE: CorrelatedColorTemperature,
     PropertyID.PRESENT_DEVICE_INPUT_POWER: Power,
     PropertyID.PRESENT_DEVICE_OPERATING_EFFICIENCY: Percentage8,
