@@ -38,11 +38,12 @@ from construct import (
     Padding,
     Select,
     Struct,
-    Switch,
     this,
 )
 
-from bluetooth_mesh.messages.util import DictAdapter, EnumAdapter, Opcode
+from bluetooth_mesh.messages.util import DictAdapter, EnumAdapter
+from bluetooth_mesh.messages.util import EnumSwitch as Switch
+from bluetooth_mesh.messages.util import Opcode, SwitchStruct
 
 
 class DebugOpcode(IntEnum):
@@ -111,15 +112,15 @@ LastFault = Struct(
     "fault" / GreedyString("utf8")
 )
 
+SystemStat = Struct(
+    "name" / PaddedString(8, "utf8"),
+    "high_water_mark" / Int16ul,
+    "rfu" / Padding(4),
+)
+
 SystemStats = Struct(
     "stats" / DictAdapter(
-        GreedyRange(
-            Struct(
-                "name" / PaddedString(8, "utf8"),
-                "high_water_mark" / Int16ul,
-                "rfu" / Padding(4),
-            )
-        ),
+        GreedyRange(SystemStat),
         key=this.name,
         value=this.high_water_mark
     )
@@ -145,42 +146,46 @@ GarbageCollectorCounter = Struct(
     "counter" / Int16ul
 )
 
+ArapSize16 = Struct(
+    "capacity" / Int16ul,
+    "size" / Int16ul
+)
+
+ArapSize8 = Struct(
+    "capacity" / Int8ul,
+    "size" / Int8ul
+)
+
 ArapSize = Select(
-    Struct(
-        "capacity" / Int16ul,
-        "size" / Int16ul
-    ),
-    Struct(
-        "capacity" / Int8ul,
-        "size" / Int8ul
-    )
+    new=ArapSize16,
+    old=ArapSize8,
 )
 
 ArapContentGet = Struct(
     "page" / Int8ul
 )
 
+ArapNode = ByteSwapped(
+    BitStruct(
+        "sequence" / Bytewise(Int24ub),
+        "ivi" / BitsInteger(1),
+        "address" / BitsInteger(15),
+    )
+)
+
 ArapContent = Struct(
     "current_page" / Int8ul,
     "last_page" / Int8ul,
     "nodes" / DictAdapter(
-        GreedyRange(
-            ByteSwapped(
-                BitStruct(
-                    "sequence" / Bytewise(Int24ub),
-                    "ivi" / BitsInteger(1),
-                    "address" / BitsInteger(15),
-                )
-            )
-        ),
+        GreedyRange(ArapNode),
         key=this.address,
         value=[this.ivi, this.sequence]
     )
 )
 
-DebugPayload = Struct(
+DebugParams = SwitchStruct(
     "subopcode" / EnumAdapter(Int8ul, DebugSubOpcode),
-    "data" / Default(Switch(
+    "payload" / Switch(
         this.subopcode,
         {
             DebugSubOpcode.RSSI_THRESHOLD_SET: RssiThreshold,
@@ -203,12 +208,17 @@ DebugPayload = Struct(
             DebugSubOpcode.ARAP_LIST_SIZE_STATUS: ArapSize,
             DebugSubOpcode.ARAP_LIST_CONTENT_GET: ArapContentGet,
             DebugSubOpcode.ARAP_LIST_CONTENT_STATUS: ArapContent,
-        },
-    ), None)
+        }
+    )
 )
 
-DebugMessage = Struct(
-    "opcode" / Const(DebugOpcode.SILVAIR_DEBUG, Opcode(DebugOpcode)),
-    "params" / DebugPayload
+DebugMessage = SwitchStruct(
+    "opcode" / Opcode(DebugOpcode),
+    "params" / Switch(
+        this.opcode,
+        {
+            DebugOpcode.SILVAIR_DEBUG: DebugParams,
+        }
+    )
 )
 # fmt: on
