@@ -35,9 +35,11 @@ from construct import (
     Embedded,
     Enum,
     ExprValidator,
+    Float32b,
     Int8ub,
     Int16ub,
     Int24ub,
+    Int32ub,
     Rebuild,
     Restreamed,
     Select,
@@ -100,6 +102,9 @@ def EnumAdapter(subcon, enum):
         ENUM = enum
 
     class _EnumAdapter(Adapter):
+        type = enum
+        _enum = enum
+
         def _decode(self, obj, context, path):
             if obj not in enum._value2member_map_:
                 raise ValidationError("object failed validation: %s" % (obj,))
@@ -122,6 +127,7 @@ def EnumAdapter(subcon, enum):
 def LogAdapter(subcon, *, max_value=None, infinity=False):
     class _LogAdapter(Adapter):
         MAX_TYPE_VALUE = int(math.pow(2, subcon.length * 8) - 1)
+        _subcon = subcon
 
         def _decode(self, obj, context, path):
             if obj == 0:
@@ -173,7 +179,19 @@ def RangeValidator(subcon, *, min_value=None, max_value=None):
     return ExprValidator(subcon, validate_range)
 
 
-def EmbeddedBitStruct(name, *args, reversed=False):
+class FieldAdapter(Adapter):
+    def __init__(self, subcon, field):
+        self._subcon = field
+        super().__init__(subcon)
+
+    def _decode(self, obj, content, path):
+        return obj
+
+    def _encode(self, obj, content, path):
+        return obj
+
+
+def EmbeddedBitStruct(name, *fields, reversed=False):
     """
     Emulates BitStruct embedding:
         - for parsing, adds Computed accessor fields to the parent construct,
@@ -181,19 +199,23 @@ def EmbeddedBitStruct(name, *args, reversed=False):
 
     NOTE: This is a hack. Do not use unless you absolutely have to.
     """
-    bit_struct = BitStruct(*args)
+    bit_struct = BitStruct(*fields)
 
     if reversed:
         bit_struct = Reversed(bit_struct)
 
-    bit_struct.__construct_doc__ = Embedded(Struct(*args))
+    bit_struct.__construct_doc__ = Embedded(Struct(*fields))
 
     return (name / Rebuild(bit_struct, dict),) + tuple(
-        i.name / Computed(this[name][i.name]) for i in args if i.name is not None
+        field.name / FieldAdapter(Computed(this[name][field.name]), field)
+        for field in fields
+        if field.name is not None
     )
 
 
 class Opcode(Construct):
+    subcon = Int32ub
+
     def __init__(self, type=int):
         super().__init__()
         self.type = type
@@ -241,6 +263,8 @@ class Opcode(Construct):
 
 
 class DefaultCountValidator(Adapter):
+    _subcon = Float32b
+
     def __init__(self, subcon, rounding=None, resolution=1.0):
         super().__init__(subcon)
         self.rounding = rounding
@@ -263,22 +287,6 @@ class DefaultCountValidator(Adapter):
             return round(obj / self.resolution)
 
 
-# TODO: is there a better way to calculate size of a nested struct?
-def SwitchLength(type, switch, mapping, path, default=0):
-    def rebuild(type, v):
-        return Rebuild(type, lambda x: len(v.build(path(x))))
-
-    return Switch(
-        switch,
-        {k: rebuild(type, v) for k, v in mapping.items()},
-        default=Rebuild(type, lambda x: default),
-    )
-
-
-def RebuildLength(type, subcons, path):
-    return Rebuild(type, lambda x: len(subcons.build(path(x))))
-
-
 class MacAddressAdapter(Adapter):
     def _decode(self, obj, context, path):
         return ":".join(map("{:02x}".format, obj))
@@ -321,26 +329,123 @@ class DictAdapter(Adapter):
                 yield {key_name: k, value_name: v}
 
 
-class OpcodeMessage(Construct):
-    OPCODE = Opcode()
+class AliasedContainer(Container):
+    ALIAS = None
+    ORIGINAL = None
 
-    def __init__(self, opcodes):
-        super().__init__()
-        self.opcodes = {k: v.compile() for k, v in opcodes.items()}
+    def __getattr__(self, name):
+        if name == self.ORIGINAL:
+            name = self.ALIAS
 
-    def _parse(self, stream, context, path):
-        opcode = self.OPCODE._parse(stream, context, path)
-        params = self.opcodes[opcode]._parse(stream, context, path)
+        return super().__getattr__(name)
 
-        return Container(opcode=opcode, params=params)
+    def __getitem__(self, name):
+        if name == self.ORIGINAL:
+            name = self.ALIAS
 
-    def _build(self, obj, stream, context, path):
-        opcode = obj["opcode"]
+        return super().__getitem__(name)
 
-        self.OPCODE._build(opcode, stream, context, path)
-        self.opcodes[opcode]._build(obj["params"], stream, context, path)
 
+class EnumSwitch(Switch):
+    def _emitparse(self, code):
+        fname = "factory_%s" % code.allocateId()
+        code.append(
+            "%s = {%s}"
+            % (
+                fname,
+                ", ".join(
+                    "%r : lambda io,this: %s" % (int(key), sc._compileparse(code))
+                    for key, sc in self.cases.items()
+                ),
+            )
+        )
+
+        defaultfname = "compiled_%s" % code.allocateId()
+        code.append(
+            "%s = lambda io,this: %s"
+            % (
+                defaultfname,
+                self.default._compileparse(code),
+            )
+        )
+        return "%s.get(%s, %s)(io, this)" % (fname, self.keyfunc, defaultfname)
+
+
+class SwitchStruct(Adapter):
+    def __init__(self, key, switch):
+        super().__init__(Struct(key, switch))
+        self.key = key
+        self.switch = switch
+        self._subcon = Struct(key, switch.subcon)
+
+    def _decode(self, obj, context, path):
+        key = self.switch.keyfunc(obj)
+
+        try:
+            value = obj[self.switch.name]
+        except KeyError:
+            value = obj[key.name.lower()]
+
+        class _Container(AliasedContainer):
+            ORIGINAL = self.switch.name
+            ALIAS = key.name.lower()
+
+        return _Container({self.key.name: key, key.name.lower(): value})
+
+    def _encode(self, obj, context, path):
+        keytype = self.key.subcon.type
+        key = keytype(self.switch.keyfunc(obj))
+
+        try:
+            value = obj[key.name.lower()]
+        except KeyError:
+            value = obj[self.switch.name]
+
+        return Container({self.key.name: key, self.switch.name: value})
+
+    def _emitparse(self, code):
+        struct = self.subcon
+        keytype = self.key.subcon.type
+
+        fname = f"parse_struct_{code.allocateId()}"
+        block = f"""
+            def {fname}(io, this):
+                from {keytype.__module__} import {keytype.__name__}
+
+                key = {self.key._compileparse(code)}
+                key_name = {keytype.__name__}(key).name.lower()
+
+                result = Container()
+                this = Container(_ = this, _params = this['_params'], _root = None, _parsing = True, _building = False, _sizing = False, _subcons = None, _io = io, _index = this.get('_index', None))
+                this['_root'] = this['_'].get('_root', this)
+                try:
+                    result[{self.key.name!r}] = this[{self.key.name!r}] = key
+                    result[key_name] = this[key_name] = {self.switch._compileparse(code)}
+                except StopFieldError:
+                    pass
+                return result
+        """
+        code.append(block)
+        return f"{fname}(io, this)"
+
+
+class NameAdapter(Adapter):
+    def _decode(self, obj, context, path):
+        obj._name = self.subcon.name
         return obj
 
-    def _sizeof(self, context, path):
-        raise SizeofError
+    def _encode(self, obj, context, path):
+        return obj.get(self.subcon.name, obj)
+
+
+class NamedSelect(Adapter):
+    def __init__(self, **subconskw):
+        subcons = list(NameAdapter(k / v.compile()) for k, v in subconskw.items())
+        super().__init__(Select(*subcons))
+        self._subcon = Select(**subconskw)
+
+    def _decode(self, obj, context, path):
+        return obj
+
+    def _encode(self, obj, context, path):
+        return obj
